from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from typing import List


class MariaDBPublicAccessDisabled(BaseResourceCheck):
    def __init__(self):
        name = "Ensure 'public network access enabled' is set to 'False' for MariaDB servers"
        id = "CKV_AZURE_48"
        supported_resources = ['azurerm_mariadb_server']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        # Whether or not public network access is allowed for this server. Defaults to true. Which is not optimal
        if 'public_network_access_enabled' not in conf or conf['public_network_access_enabled'][0]:
            return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['public_network_access_enabled']


check = MariaDBPublicAccessDisabled()



from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar, Generic

if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.local_graph import LocalGraph

T = TypeVar("T")


class DBConnector(Generic[T]):
    def save_graph(self, local_graph: LocalGraph) -> T:
        pass

    def get_reader_endpoint(self) -> T:
        pass

    def get_writer_endpoint(self) -> T:
        pass

    def disconnect(self) -> None:
        pass



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class NetworkIPsecAlgorithms(BaseResourceCheck):
    def __init__(self):
        name = "Ensure IPsec profiles do not specify use of insecure encryption algorithms"
        id = "CKV_PAN_11"
        supported_resources = ['panos_ipsec_crypto_profile','panos_panorama_ipsec_crypto_profile']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
    
        # Check there are encryptions defined in the resource
        if 'encryptions' in conf:

            # Report the area of evaluation
            self.evaluated_keys = ['encryptions']

            # Get all the algorithms
            algorithms = conf.get('encryptions')

            # Iterate over each algorithm, as multiple can be defined in "encryptions"
            for algo in algorithms:

                # Check for insecure algorithms, including null as a string (not a null value)
                if algo[0] in ('des', '3des', 'aes-128-cbc', 'aes-192-cbc', 'aes-256-cbc', 'null'):

                    # Fail if any insecure algorithms are defined for use
                    return CheckResult.FAILED

            # If no fails have been found, this is a pass
            return CheckResult.PASSED

        # If the mandatory "encryptions" attribute is not defined, this is not valid, and will fail during Terraform plan stage, and should therefore be a fail
        return CheckResult.FAILED

check = NetworkIPsecAlgorithms()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class LaunchConfigurationEBSEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = (
            "Ensure all data stored in the Launch configuration or instance Elastic Blocks Store "
            "is securely encrypted"
        )
        id = "CKV_AWS_8"
        supported_resources = ("aws_launch_configuration", "aws_instance")
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
            Looks for encryption configuration at launch configuration:
            https://www.terraform.io/docs/providers/aws/r/launch_configuration.html or
            https://www.terraform.io/docs/providers/aws/d/instance.html
        :param conf: aws_launch_configuration configuration
        :return: <CheckResult>
        """
        # for key in conf.keys():
        #     # If present, the encrypted flag will be determined by the snapshot
        #     # Note: checkov does not know if snapshot is encrypted, so we default to PASSED

        self.evaluated_keys = ["root_block_device"]
        root = conf.get("root_block_device")
        if not root or not root[0]:
            # Issue 496 - TF will create unencrypted EBS root by default if whole root_block_device block is omitted.
            return CheckResult.FAILED
        self.evaluated_keys.append("ebs_block_device")
        blocks = conf.get("ebs_block_device") or []

        allblocks = root + blocks

        if not allblocks:
            return CheckResult.UNKNOWN

        for block in allblocks:
            if isinstance(block, dict) and not block.get("encrypted") == [True]:
                if not block.get("snapshot_id"):
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = LaunchConfigurationEBSEncryption()



import ast
from typing import Generator, Tuple, List

from . import __version__ as version
from . model_parts_info import get_model_parts_info
from . class_members_errors import get_class_members_errors


class ClassAttributesChecker:

    name = 'flake8-class-attributes'
    version = version
    options = None

    def __init__(self, tree, filename: str):
        self.filename = filename
        self.tree = tree

    @classmethod
    def add_options(cls, parser) -> None:
        parser.add_option(
            '--use-class-attributes-order-strict-mode',
            action='store_true',
            parse_from_config=True,
            help='Require more strict order of private class members',
        )
        parser.add_option(
            '--class-attributes-order',
            comma_separated_list=True,
            parse_from_config=True,
            help='Comma-separated list of class attributes to '
                 'configure order manually',
        )

    @classmethod
    def parse_options(cls, options: str) -> None:
        cls.options = options

    def run(self) -> Generator[Tuple[int, int, str, type], None, None]:
        classes = [n for n in ast.walk(self.tree) if isinstance(n, ast.ClassDef)]
        errors: List[Tuple[int, int, str]] = []

        for class_def in classes:
            model_parts_info = get_model_parts_info(class_def)
            errors += get_class_members_errors(model_parts_info, class_def)

        for lineno, col_offset, error_msg in errors:
            yield lineno, col_offset, error_msg, type(self)



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CloudtrailEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure CloudTrail logs are encrypted at rest using KMS CMKs"
        id = "CKV_AWS_35"
        supported_resources = ("AWS::CloudTrail::Trail",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/KMSKeyId"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = CloudtrailEncryption()



from __future__ import annotations

import concurrent
import hashlib
from typing import Any, Callable
import concurrent.futures


def calculate_hash(data: Any) -> str:
    sha256 = hashlib.sha256(str(data).encode("utf-8"))
    return sha256.hexdigest()


def join_trimmed_strings(char_to_join: str, str_lst: list[str], num_to_trim: int) -> str:
    return char_to_join.join(str_lst[: len(str_lst) - num_to_trim])


def run_function_multithreaded(
    func: Callable[..., Any], data: list[list[Any]], max_group_size: int, num_of_workers: int | None = None
) -> None:
    groups_of_data = [data[i : i + max_group_size] for i in range(0, len(data), max_group_size)]
    if not num_of_workers:
        num_of_workers = len(groups_of_data)
    if num_of_workers > 0:
        with concurrent.futures.ThreadPoolExecutor(max_workers=num_of_workers) as executor:
            futures = {executor.submit(func, data_group): data_group for data_group in groups_of_data}
            wait_result = concurrent.futures.wait(futures)
            if wait_result.not_done:
                raise Exception(f"failed to perform {func.__name__}")
            for future in futures:
                try:
                    future.result()
                except Exception:
                    raise


def filter_sub_keys(key_list: list[str]) -> list[str]:
    filtered_key_list = []
    for key in key_list:
        if not any(other_key != key and other_key.startswith(key) for other_key in key_list):
            filtered_key_list.append(key)
    return filtered_key_list



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AzureScaleSetPassword(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Azure linux scale set does not use basic authentication(Use SSH Key Instead)"
        id = "CKV_AZURE_49"
        supported_resources = ['azurerm_linux_virtual_machine_scale_set']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "disable_password_authentication/[0]"


check = AzureScaleSetPassword()



#
# No change required normally except to possibly switch it to json
# eg. from checkov.json_doc.base_registry import Registry
# 
from checkov.yaml_doc.base_registry import Registry

registry = Registry()



from __future__ import annotations

import json
import requests
import logging
import time
import os
from typing import Any, TYPE_CHECKING, cast

from checkov.common.bridgecrew.bc_source import SourceType
from checkov.common.util.consts import DEV_API_GET_HEADERS, DEV_API_POST_HEADERS
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.version import version as checkov_version

if TYPE_CHECKING:
    from requests import Response


logger = logging.getLogger(__name__)


def normalize_prisma_url(url: str) -> str | None:
    """ Correct common Prisma Cloud API URL misconfigurations """
    if not url:
        return None
    return url.lower().replace('//app', '//api').replace('http:', 'https:').rstrip('/')


def extract_error_message(response: requests.Response) -> str:
    if response.content:
        try:
            content = json.loads(response.content)
            if 'message' in content:
                return cast(str, content['message'])
        except Exception:
            logging.debug(f'Failed to parse the response content: {response.content.decode()}')

    return response.reason


def get_auth_header(token: str) -> dict[str, str]:
    return {
        'Authorization': token
    }


def get_version_headers(client: str, client_version: str) -> dict[str, str]:
    return {
        'x-api-client': client,
        'x-api-version': client_version,
        'x-api-checkov-version': checkov_version
    }


def get_user_agent_header() -> dict[str, str]:
    return {'User-Agent': f'checkov/{checkov_version}'}


def get_default_get_headers(client: SourceType, client_version: str) -> dict[str, Any]:
    return merge_dicts(DEV_API_GET_HEADERS, get_version_headers(client.name, client_version), get_user_agent_header())


def get_default_post_headers(client: SourceType, client_version: str) -> dict[str, Any]:
    return merge_dicts(DEV_API_POST_HEADERS, get_version_headers(client.name, client_version), get_user_agent_header())


def request_wrapper(
    method: str,
    url: str,
    headers: dict[str, Any],
    data: Any | None = None,
    json: dict[str, Any] | None = None,
    should_call_raise_for_status: bool = False
) -> Response | None:
    # using of "retry" mechanism for 'requests.request' due to unpredictable 'ConnectionError' and 'HttpError'
    # instances that appears from time to time.
    # 'ConnectionError' instances that appeared:
    # * 'Connection aborted.', ConnectionResetError(104, 'Connection reset by peer').
    # * 'Connection aborted.', OSError(107, 'Socket not connected').
    # 'ConnectionError' instances that appeared:
    # * 403 Client Error: Forbidden for url.
    # * 504 Server Error: Gateway Time-out for url.

    request_max_tries = int(os.getenv('REQUEST_MAX_TRIES', 3))
    sleep_between_request_tries = float(os.getenv('SLEEP_BETWEEN_REQUEST_TRIES', 1))

    for i in range(request_max_tries):
        try:
            response = requests.request(method, url, headers=headers, data=data, json=json)
            if should_call_raise_for_status:
                response.raise_for_status()
            return response
        except requests.exceptions.ConnectionError as connection_error:
            logging.error(f"Connection error on request {method}:{url},\ndata:\n{data}\njson:{json}\nheaders:{headers}")
            if i != request_max_tries - 1:
                time.sleep(sleep_between_request_tries * (i + 1))
                continue
            raise connection_error
        except requests.exceptions.HTTPError as http_error:
            logging.error(f"HTTP error on request {method}:{url},\ndata:\n{data}\njson:{json}\nheaders:{headers}")
            status_code = http_error.response.status_code
            if (status_code >= 500 or status_code == 403) and i != request_max_tries - 1:
                time.sleep(sleep_between_request_tries * (i + 1))
                continue
            raise http_error

    return None



from __future__ import annotations

from typing import Any

from checkov.bicep.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class StorageAccountDefaultNetworkAccessDeny(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure default network access rule for Storage Accounts is set to deny"
        id = "CKV_AZURE_35"
        supported_resources = ("Microsoft.Storage/storageAccounts",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/networkAcls/defaultAction"

    def get_expected_value(self) -> Any:
        return "Deny"


check = StorageAccountDefaultNetworkAccessDeny()



import json
import logging
import os
from typing import Dict, List, Tuple, Any, Optional
import dpath

from checkov.terraform.context_parsers.registry import parser_registry
from checkov.terraform.plan_parser import parse_tf_plan
from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.runner_filter import RunnerFilter
from checkov.common.parsers.node import DictNode


def create_definitions(
    root_folder: str,
    files: Optional[List[str]] = None,
    runner_filter: RunnerFilter = RunnerFilter(),
    out_parsing_errors: Dict[str, str] = {}
) -> Tuple[Dict[str, DictNode], Dict[str, List[Tuple[int, str]]]]:
    if root_folder:
        files = [] if not files else files
        for root, d_names, f_names in os.walk(root_folder):
            filter_ignored_paths(root, d_names, runner_filter.excluded_paths)
            filter_ignored_paths(root, f_names, runner_filter.excluded_paths)
            for file in f_names:
                file_ending = os.path.splitext(file)[1]
                if file_ending == '.json':
                    try:
                        with open(f'{root}/{file}') as f:
                            content = json.load(f)
                        if isinstance(content, dict) and content.get('terraform_version'):
                            files.append(os.path.join(root, file))
                    except Exception as e:
                        logging.debug(f'Failed to load json file {root}/{file}, skipping')
                        logging.debug('Failure message:')
                        logging.debug(e, stack_info=True)
                        out_parsing_errors[file] = str(e)
    tf_definitions = {}
    definitions_raw = {}
    if files:
        files = [os.path.realpath(file) for file in files]
        for file in files:
            if file.endswith(".json"):
                current_tf_definitions, current_definitions_raw = parse_tf_plan(file, out_parsing_errors)
                if current_tf_definitions and current_definitions_raw:
                    tf_definitions[file] = current_tf_definitions
                    definitions_raw[file] = current_definitions_raw
            else:
                logging.debug(f'Failed to load {file} as is not a .json file, skipping')
    return tf_definitions, definitions_raw


def build_definitions_context(definitions: Dict[str, DictNode], definitions_raw: Dict[str, List[Tuple[int, str]]]) -> \
        Dict[str, Dict[str, Any]]:
    definitions_context = {}
    block_type = 'resource'
    for full_file_path, definition in definitions.items():
        entities = definition.get(block_type, [])
        for entity in entities:
            context_parser = parser_registry.context_parsers[block_type]
            definition_path = context_parser.get_entity_context_path(entity)
            entity_id = ".".join(definition_path)
            # Entity can exist only once per dir, for file as well
            entity_context = get_entity_context(definitions, definitions_raw, definition_path, full_file_path)
            dpath.new(definitions_context, [full_file_path, entity_id], entity_context)
    return definitions_context


def get_entity_context(definitions, definitions_raw, definition_path, full_file_path):
    # return self.context.get(full_file_path, {})
    entity_context = {}

    if full_file_path not in definitions:
        logging.debug(
            f'Tried to look up file {full_file_path} in TF plan entity definitions, but it does not exist')
        return entity_context

    for resource in definitions.get(full_file_path, {}).get('resource', []):
        resource_type = definition_path[0]
        if resource_type in resource.keys():
            resource_name = definition_path[1]
            if resource_name in resource[resource_type].keys():
                resource_defintion = resource[resource_type][resource_name]
                entity_context['start_line'] = resource_defintion['start_line'][0]
                entity_context['end_line'] = resource_defintion['end_line'][0]
                entity_context["code_lines"] = definitions_raw[full_file_path][
                                               entity_context["start_line"]: entity_context["end_line"]]
                entity_context['address'] = resource_defintion['__address__']
                return entity_context
    return entity_context



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class GlueSecurityConfiguration(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Glue Security Configuration Encryption is enabled"
        id = "CKV_AWS_99"
        supported_resources = ['aws_glue_security_configuration']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'encryption_configuration' not in conf:
            return CheckResult.FAILED
        data_conf = conf['encryption_configuration'][0]
        cloudwatch_encrypted = False
        job_bookmarks_encrypted = False
        s3_encrypted = False
        if 'cloudwatch_encryption' in data_conf:
            enc_res = data_conf['cloudwatch_encryption'][0]
            if 'cloudwatch_encryption_mode' in enc_res and 'kms_key_arn' in enc_res \
                    and enc_res['cloudwatch_encryption_mode'][0] == "SSE-KMS":
                cloudwatch_encrypted = True
                self.evaluated_keys = ['encryption_configuration/[0]/cloudwatch_encryption/[0]/kms_key_arn',
                                       'encryption_configuration/[0]/cloudwatch_encryption/[0]/'
                                       'cloudwatch_encryption_mode']
            else:
                self.evaluated_keys = ['encryption_configuration/[0]/cloudwatch_encryption']

        if 'job_bookmarks_encryption' in data_conf:
            enc_res = data_conf['job_bookmarks_encryption'][0]
            if 'job_bookmarks_encryption_mode' in enc_res and 'kms_key_arn' in enc_res \
                    and enc_res['job_bookmarks_encryption_mode'][0] == "CSE-KMS":
                job_bookmarks_encrypted = True
                self.evaluated_keys.append(
                    'encryption_configuration/[0]/job_bookmarks_encryption/[0]/job_bookmarks_encryption_mode')
                self.evaluated_keys.append('encryption_configuration/[0]/job_bookmarks_encryption/[0]/kms_key_arn')
            else:
                self.evaluated_keys.append('encryption_configuration/[0]/job_bookmarks_encryption')

        if 's3_encryption' in data_conf:
            enc_res = data_conf['s3_encryption'][0]
            if 's3_encryption_mode' in enc_res and enc_res['s3_encryption_mode'][0] != "DISABLED":
                s3_encrypted = True
                self.evaluated_keys.append('encryption_configuration/[0]/s3_encryption/[0]/s3_encryption_mode')
            else:
                self.evaluated_keys.append('encryption_configuration/[0]/s3_encryption')

        if cloudwatch_encrypted and job_bookmarks_encrypted and s3_encrypted:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = GlueSecurityConfiguration()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DLMEventsCrossRegionEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DLM cross region events are encrypted"
        id = "CKV_AWS_253"
        supported_resources = ['aws_dlm_lifecycle_policy']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("policy_details") and isinstance(conf.get("policy_details"), list):
            policy = conf.get("policy_details")[0]
            if policy.get("action") and isinstance(policy.get("action"), list):
                actions = policy.get("action")
                for idx, action in enumerate(actions):
                    if action.get("cross_region_copy") and isinstance(action.get("cross_region_copy"), list):
                        cross = action.get("cross_region_copy")[0]
                        if cross.get("encryption_configuration") and isinstance(cross.get("encryption_configuration"), list):
                            config = cross.get("encryption_configuration")[0]
                            if config.get("encryption") == [True]:
                                return CheckResult.PASSED
                        self.evaluated_keys = [f"policy_details/action/{idx}/cross_region_copy/encryption_configuration"]
                        return CheckResult.FAILED
                    return CheckResult.UNKNOWN
                return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = DLMEventsCrossRegionEncryption()



from checkov.common.vcs.vcs_schema import VCSSchema


class OrgSecuritySchema(VCSSchema):
    def __init__(self):
        schema = {
            "type": "object",
            "properties": {
                "data": {
                    "type": "object",
                    "properties": {
                        "organization": {
                            "type": "object",
                            "properties": {
                                "name": {
                                    "type": "string"
                                },
                                "login": {
                                    "type": "string"
                                },
                                "description": {
                                    "type": "string"
                                },
                                "ipAllowListEnabledSetting": {
                                    "type": "string"
                                },
                                "ipAllowListForInstalledAppsEnabledSetting": {
                                    "type": "string"
                                },
                                "requiresTwoFactorAuthentication": {
                                    "type": "boolean"
                                },
                                "samlIdentityProvider": {
                                    "type": "object",
                                    "properties": {
                                        "ssoUrl": {
                                            "type": "string"
                                        }
                                    }
                                }
                            },
                            "required": [
                                "name",
                                "login",
                                "description",
                                "ipAllowListEnabledSetting",
                                "ipAllowListForInstalledAppsEnabledSetting",
                                "requiresTwoFactorAuthentication"]
                        }
                    },
                    "required": [
                        "organization"
                    ]
                }
            },
            "required": [
                "data"
            ]
        }
        super().__init__(schema=schema)


schema = OrgSecuritySchema()



from __future__ import annotations

import os
import re
from pathlib import Path
from typing import Union, List, Tuple, Optional, Dict, Any

from colorama import init, Fore, Style
from termcolor import colored

from checkov.common.bridgecrew.severities import Severity
from checkov.common.models.enums import CheckResult
from checkov.common.typing import _CheckResult
from checkov.common.util.file_utils import convert_to_unix_path
from checkov.common.util.type_forcers import force_int

init(autoreset=True)

DEFAULT_SEVERITY = "none"  # equivalent to a score of 0.0 in the CVSS v3.0 Ratings

OUTPUT_CODE_LINE_LIMIT = force_int(os.getenv('CHECKOV_OUTPUT_CODE_LINE_LIMIT')) or 50

ANSI_COLORS_DISABLED = bool(os.getenv('ANSI_COLORS_DISABLED'))

class Record:
    def __init__(
        self,
        check_id: str,
        check_name: str,
        check_result: _CheckResult,
        code_block: List[Tuple[int, str]],
        file_path: str,
        file_line_range: List[int],
        resource: str,
        evaluations: Optional[Dict[str, Any]],
        check_class: str,
        file_abs_path: str,
        entity_tags: Optional[Dict[str, str]] = None,
        caller_file_path: Optional[str] = None,
        caller_file_line_range: Optional[Tuple[int, int]] = None,
        bc_check_id: Optional[str] = None,
        resource_address: Optional[str] = None,
        severity: Optional[Severity] = None,
        bc_category: Optional[str] = None,
        benchmarks: dict[str, list[str]] | None = None,
        description: Optional[str] = None,
        short_description: Optional[str] = None,
        vulnerability_details: Optional[Dict[str, Any]] = None,
        connected_node: Optional[Dict[str, Any]] = None
    ) -> None:
        """
        :param evaluations: A dict with the key being the variable name, value being a dict containing:
                             - 'var_file'
                             - 'value'
                             - 'definitions', a list of dicts which contain 'definition_expression'
        """
        self.check_id = check_id
        self.bc_check_id = bc_check_id
        self.check_name = check_name
        self.check_result = check_result
        self.code_block = code_block
        self.file_path = file_path
        self.file_abs_path = file_abs_path
        self.repo_file_path = self._determine_repo_file_path(file_abs_path)
        self.file_line_range = file_line_range
        self.resource = resource
        self.evaluations = evaluations
        self.check_class = check_class
        self.fixed_definition = None
        self.entity_tags = entity_tags
        self.caller_file_path = caller_file_path  # When created from a module
        self.caller_file_line_range = caller_file_line_range  # When created from a module
        self.resource_address = resource_address
        self.severity = severity
        self.bc_category = bc_category
        self.benchmarks = benchmarks
        self.description = description  # used by SARIF output
        self.short_description = short_description  # used by SARIF output
        self.vulnerability_details = vulnerability_details  # Stores package vulnerability details
        self.connected_node = connected_node
        self.guideline: str | None = None

    @staticmethod
    def _determine_repo_file_path(file_path: Union[str, "os.PathLike[str]"]) -> str:
        # matches file paths given in the BC platform and should always be a unix path
        repo_file_path = Path(file_path)
        if Path.cwd().drive == repo_file_path.drive:
            return convert_to_unix_path(f"/{os.path.relpath(repo_file_path)}").replace("/..", "")

        return f"/{'/'.join(repo_file_path.parts[1:])}"

    def set_guideline(self, guideline: Optional[str]) -> None:
        self.guideline = guideline

    @staticmethod
    def _trim_special_chars(expression: str) -> str:
        return "".join(re.findall(re.compile(r"[^ ${\}]+"), expression))

    def _is_expression_in_code_lines(self, expression: str) -> bool:
        stripped_expression = self._trim_special_chars(expression)
        return any(stripped_expression in self._trim_special_chars(line) for (_, line) in self.code_block)

    @staticmethod
    def _code_line_string(code_block: List[Tuple[int, str]], colorized: bool = True) -> str:
        code_output = []
        color_codes = (Fore.WHITE if colorized else "", Fore.YELLOW if colorized else "")
        last_line_number_len = len(str(code_block[-1][0]))

        if len(code_block) >= OUTPUT_CODE_LINE_LIMIT:
            return f'\t\t{color_codes[1]}Code lines for this resource are too many. ' \
                   f'Please use IDE of your choice to review the file.'

        for line_num, line in code_block:
            spaces = " " * (last_line_number_len - len(str(line_num)))
            if line.lstrip().startswith("#"):
                code_output.append(f"\t\t{color_codes[0]}{line_num}{spaces} | {line}")
            else:
                code_output.append(f"\t\t{color_codes[0]}{line_num}{spaces} | {color_codes[1]}{line}")
        return "".join(code_output)

    def to_string(self, compact: bool = False, use_bc_ids: bool = False) -> str:
        status = ""
        evaluation_message = ""
        status_color = "white"
        suppress_comment = ""
        if self.check_result["result"] == CheckResult.PASSED:
            status = CheckResult.PASSED.name
            status_color = "green"
        elif self.check_result["result"] == CheckResult.FAILED:
            status = CheckResult.FAILED.name
            status_color = "red"
        elif self.check_result["result"] == CheckResult.SKIPPED:
            status = CheckResult.SKIPPED.name
            status_color = "blue"
            suppress_comment = "\tSuppress comment: {}\n".format(self.check_result["suppress_comment"])

        check_message = colored('Check: {}: "{}"\n'.format(self.get_output_id(use_bc_ids), self.check_name), "white")
        guideline_message = ""
        if self.guideline:
            guideline_message = (
                "\tGuide: "
                + Style.BRIGHT
                + colored(f"{self.guideline}\n", "blue", attrs=["underline"])
                + Style.RESET_ALL
            )

        severity_message = f'\tSeverity: {self.severity.name}\n' if self.severity else ''

        file_details = colored(
            "\tFile: {}:{}\n".format(self.file_path, "-".join([str(x) for x in self.file_line_range])), "magenta"
        )
        code_lines = ""
        if self.code_block:
            code_lines = "\n{}\n".format("".join([self._code_line_string(self.code_block, not(ANSI_COLORS_DISABLED))]))
        caller_file_details = ""
        if self.caller_file_path and self.caller_file_line_range:
            caller_file_details = colored(
                "\tCalling File: {}:{}\n".format(
                    self.caller_file_path, "-".join([str(x) for x in self.caller_file_line_range])
                ),
                "magenta",
            )
        if self.evaluations:
            for (var_name, var_evaluations) in self.evaluations.items():
                var_file = var_evaluations["var_file"]
                var_definitions = var_evaluations["definitions"]
                for definition_obj in var_definitions:
                    definition_expression = definition_obj["definition_expression"]
                    if self._is_expression_in_code_lines(definition_expression):
                        evaluation_message = evaluation_message + colored(
                            f'\tVariable {colored(var_name, "yellow")} (of {var_file}) evaluated to value "{colored(var_evaluations["value"], "yellow")}" '
                            f'in expression: {colored(definition_obj["definition_name"] + " = ", "yellow")}{colored(definition_obj["definition_expression"], "yellow")}\n',
                            "white",
                        )

        status_message = colored("\t{} for resource: {}\n".format(status, self.resource), status_color)
        if self.check_result["result"] == CheckResult.FAILED and code_lines and not compact:
            return f"{check_message}{status_message}{severity_message}{file_details}{caller_file_details}{guideline_message}{code_lines}{evaluation_message}"

        if self.check_result["result"] == CheckResult.SKIPPED:
            return f"{check_message}{status_message}{severity_message}{suppress_comment}{file_details}{caller_file_details}{guideline_message}"
        else:
            return f"{check_message}{status_message}{severity_message}{file_details}{caller_file_details}{evaluation_message}{guideline_message}"

    def __str__(self) -> str:
        return self.to_string()

    def get_output_id(self, use_bc_ids: bool) -> str:
        return self.bc_check_id if self.bc_check_id and use_bc_ids else self.check_id

    def get_unique_string(self) -> str:
        return f"{self.check_id}.{self.check_result}.{self.file_abs_path}.{self.file_line_range}.{self.resource}"



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.parsers.node import DictNode

class AKSDashboardDisabled(BaseResourceCheck):
    def __init__(self):
        # apiVersion 2017-08-03 = Fail - No addonProfiles option to configure
        name = "Ensure Kubernetes Dashboard is disabled"
        id = "CKV_AZURE_8"
        supported_resources = ['Microsoft.ContainerService/managedClusters']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("apiVersion") is not None:
            if conf["apiVersion"] == "2017-08-31":
                # No addonProfiles option to configure
                return CheckResult.FAILED

        properties = conf.get("properties")
        if properties is None or not isinstance(properties, DictNode):
            return CheckResult.FAILED
        addon_profiles = conf["properties"].get("addonProfiles")
        if not isinstance(addon_profiles, DictNode):
            return CheckResult.FAILED
        kube_dashboard = addon_profiles.get("kubeDashboard")
        if not isinstance(kube_dashboard, DictNode):
            return CheckResult.FAILED
        enabled = kube_dashboard.get("enabled")
        if enabled is not None and str(enabled).lower() == "false":
            return CheckResult.PASSED
        return CheckResult.FAILED

check = AKSDashboardDisabled()


from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck

class SecurityGroupRuleDescription(BaseResourceCheck):
    def __init__(self):
        name = "Ensure every security groups rule has a description"
        id = "CKV_AWS_23"
        supported_resource = ['AWS::EC2::SecurityGroup', 'AWS::EC2::SecurityGroupIngress', 'AWS::EC2::SecurityGroupEgress']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def scan_resource_conf(self, conf):
        """
            Looks for description in security group rules :
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html
        :param conf: aws_security_group configuration
        :return: <CheckResult>
        """

        if conf['Type'] == 'AWS::EC2::SecurityGroup':
            if 'Properties' in conf:
                security_group_ingress_rules = conf.get('Properties', {}).get('SecurityGroupIngress')
                if security_group_ingress_rules:
                    for rule in security_group_ingress_rules:
                        if isinstance(rule, dict) and ('Description' not in rule or not rule['Description']):
                            return CheckResult.FAILED
                security_group_egress_rules = conf.get('Properties', {}).get('SecurityGroupEgress')
                if security_group_egress_rules:
                    for rule in security_group_egress_rules:
                        if isinstance(rule, dict) and ('Description' not in rule.keys() or not rule['Description']):
                            return CheckResult.FAILED
                return CheckResult.PASSED

        elif conf['Type'] == 'AWS::EC2::SecurityGroupIngress' or conf['Type'] == 'AWS::EC2::SecurityGroupEgress':
            if 'Properties' in conf.keys() and 'Description' in conf['Properties'] and conf['Properties']['Description']:
                return CheckResult.PASSED

        return CheckResult.FAILED


check = SecurityGroupRuleDescription()



# the order is important, because it reflects the order, which will be used to download the module
from checkov.terraform.module_loading.loaders.registry_loader import RegistryLoader
from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader
from checkov.terraform.module_loading.loaders.github_loader import GithubLoader
from checkov.terraform.module_loading.loaders.bitbucket_loader import BitbucketLoader
from checkov.terraform.module_loading.loaders.github_access_token_loader import GithubAccessTokenLoader
from checkov.terraform.module_loading.loaders.bitbucket_access_token_loader import BitbucketAccessTokenLoader
from checkov.terraform.module_loading.loaders.local_path_loader import LocalPathLoader



DEFAULT_EXTERNAL_MODULES_DIR = ".external_modules"
RESOLVED_MODULE_ENTRY_NAME = "__resolved__"

DEV_API_GET_HEADERS = {
    'Accept': 'application/json'
}

DEV_API_POST_HEADERS = {
    'Accept': 'application/json',
    'Content-Type': 'application/json'
}

PARSE_ERROR_FAIL_FLAG = 'CKV_PARSE_ERROR_FAIL'



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ELBv2AccessLogs(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure the ELBv2 (Application/Network) has access logging enabled"
        id = "CKV_AWS_91"
        supported_resources = ["aws_lb", "aws_alb"]
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "access_logs/0/enabled/0"

    def scan_resource_conf(self, conf):
        if conf.get("load_balancer_type") == ["gateway"]:
            return CheckResult.UNKNOWN
        return super().scan_resource_conf(conf)


check = ELBv2AccessLogs()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class ElasticsearchEncryptionWithCMK(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure all data stored in the Elasticsearch is encrypted with a CMK"
        id = "CKV_AWS_247"
        supported_resources = ('aws_elasticsearch_domain', 'aws_opensearch_domain')
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "encrypt_at_rest/[0]/kms_key_id"

    def get_expected_value(self):
        return ANY_VALUE


check = ElasticsearchEncryptionWithCMK()



from typing import List, Any
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck

class ComputeVMSerialConsole(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure compute instance does not have serial console enabled."
        id = "CKV_YC_4"
        categories = [CheckCategories.GENERAL_SECURITY]
        supported_resources = ["yandex_compute_instance"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "metadata/[0]/serial-port-enable"

    def get_forbidden_values(self) -> List[Any]:
        return [True]

check = ComputeVMSerialConsole()


from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CodeBuildProjectEncryption(BaseResourceCheck):

    def __init__(self):
        name = "Ensure that CodeBuild Project encryption is not disabled"
        id = "CKV_AWS_78"
        supported_resources = ['aws_codebuild_project']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'artifacts' not in conf:
            return CheckResult.UNKNOWN
        artifact = force_list(conf['artifacts'])[0]
        if isinstance(artifact, dict):
            if artifact['type'] == ["NO_ARTIFACTS"]:
                self.evaluated_keys = ['artifacts/[0]/type']
                return CheckResult.UNKNOWN
            if 'encryption_disabled' in artifact and artifact['encryption_disabled'] == [True]:
                self.evaluated_keys = ['artifacts/[0]/encryption_disabled']
                return CheckResult.FAILED
        self.evaluated_keys = ['artifacts']
        return CheckResult.PASSED


check = CodeBuildProjectEncryption()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
from checkov.common.util.type_forcers import force_int


class AbsSecurityGroupUnrestrictedIngress(BaseResourceCheck):
    def __init__(self, check_id, port):
        name = "Ensure no security groups allow ingress from 0.0.0.0:0 to port %d" % port
        supported_resources = ['alicloud_security_group_rule']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def scan_resource_conf(self, conf):
        """
            Looks for configuration at security group ingress rules :
            https://registry.terraform.io/providers/aliyun/alicloud/latest/docs/resources/security_group_rule

            Return PASS if:
            - The resource is an alicloud_security_group_rule of type 'ingress' that does not violate the check.

            Return FAIL if:
            - The resource is an alicloud_security_group_rule of type 'ingress' that violates the check.

            Return UNKNOWN if:
            - the resource is an alicloud_security_group_rule of type 'egress'

        :param conf: alicloud_security_group_rule configuration
        :return: <CheckResult>
        """

        if 'type' not in conf:  # This means it's not an alicloud_security_group_rule resource.
            return CheckResult.PASSED

        rule_type = force_list(conf['type'])[0]
        if rule_type != 'ingress':
            return CheckResult.UNKNOWN
        self.evaluated_keys = ['port_range', 'cidr_ip']
        if not conf.get('port_range'):
            return CheckResult.PASSED
        if self.contains_violation(conf):
            return CheckResult.FAILED
        return CheckResult.PASSED

    def contains_violation(self, conf):
        from_port = force_int(force_list(conf.get('port_range', [{-1}]))[0].split('/')[0])
        to_port = force_int(force_list(conf.get('port_range', [{-1}]))[0].split('/')[1])

        if from_port and to_port and from_port <= self.port <= to_port:
            conf_cidr_blocks = conf.get('cidr_ip', [[]])
            cidr_blocks = force_list(conf_cidr_blocks)
            if "0.0.0.0/0" in cidr_blocks or not cidr_blocks[0]:
                return True
        return False



from checkov.kubernetes.checks.resource.base_rbac_check import BaseRbacK8sCheck, RbacOperation


class RbacBindRoleBindings(BaseRbacK8sCheck):
    def __init__(self):
        name = "Minimize Roles and ClusterRoles that grant permissions to bind RoleBindings or ClusterRoleBindings"
        id = "CKV_K8S_157"
        super().__init__(name=name, id=id)

        self.failing_operations = [
            RbacOperation(
                apigroups=["rbac.authorization.k8s.io"],
                verbs=["bind"],
                resources=["rolebindings", "clusterrolebindings"]
            ),
        ]


check = RbacBindRoleBindings()



#!/usr/bin/env python
import logging
import os
from importlib import util
from os import path

import setuptools
from setuptools import setup

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

logger = logging.getLogger(__name__)
spec = util.spec_from_file_location(
    "checkov.version", os.path.join("checkov", "version.py")
)
# noinspection PyUnresolvedReferences
mod = util.module_from_spec(spec)
spec.loader.exec_module(mod)  # type: ignore
version = mod.version  # type: ignore

setup(
    extras_require={
        "dev": [
            "pytest==5.3.1",
            "coverage==5.5",
            "coverage-badge",
            "GitPython==3.1.7",
            "bandit",
            "jsonschema",
        ]
    },
    install_requires=[
        "bc-python-hcl2==0.3.42",
        "cloudsplaining>=0.4.1",
        "deep_merge",
        "tabulate",
        "colorama",
        "termcolor",
        "junit-xml>=1.9",
        "dpath>=1.5.0,<2",
        "pyyaml>=5.4.1",
        "boto3>=1.17",
        "GitPython",
        "jmespath",
        "tqdm",
        "update_checker",
        "semantic_version",
        "packaging",
        "networkx",
        "dockerfile-parse",
        "docker",
        "configargparse",
        "argcomplete",
        "detect-secrets",
        "policyuniverse",
        "typing-extensions>=4.1.0",
        "cachetools",
        "cyclonedx-python-lib>=2.4.0",
        "click>=8.0.0",
        "aiohttp",
        "aiodns",
        "aiomultiprocess",
        "jsonpath_ng",
        "jsonschema~=3.0",
        "prettytable>=3.0.0",
        "pycep-parser==0.3.6",
        "charset-normalizer",
    ],
    license="Apache License 2.0",
    name="checkov",
    version=version,
    python_requires=">=3.7",
    description="Infrastructure as code static analysis",
    author="bridgecrew",
    author_email="meet@bridgecrew.io",
    url="https://github.com/bridgecrewio/checkov",
    packages=setuptools.find_packages(exclude=["tests*", "integration_tests*"]),
    include_package_data=True,
    package_dir={
        "checkov.bicep.checks.graph_checks": "checkov/bicep/checks/graph_checks",
        "checkov.terraform.checks.graph_checks": "checkov/terraform/checks/graph_checks",
    },
    package_data={
        "checkov": ["py.typed"],
        "checkov.bicep.checks.graph_checks": ["*.yaml"],
        "checkov.common.util.templates": ["*.jinja2"],
        "checkov.terraform.checks.graph_checks": [
            "aws/*.yaml",
            "gcp/*.yaml",
            "azure/*.yaml",
        ],
    },
    scripts=["bin/checkov", "bin/checkov.cmd"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Security",
        "Topic :: Software Development :: Build Tools",
        "Typing :: Typed",
    ],
)
